package net.kldp.OpenSigner.applet;

import java.applet.Applet;
import java.awt.Button;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.security.Key;
import java.security.KeyStore;
import java.security.Principal;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Security;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.HashMap;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;

import org.bouncycastle.jce.provider.BouncyCastleProvider;
import net.kldp.OpenSigner.utils.Base64;
import net.kldp.OpenSigner.utils.StringUtil;
import net.kldp.OpenSigner.utils.xmldsigUtil;
import netscape.javascript.JSObject;

class UserCert
{
	HashMap certInfo = null;
	
	public UserCert(Certificate[] userCert, int i)
	{
		int index_now = i - 1;
		certInfo = new HashMap();
		if (userCert[index_now] instanceof X509Certificate) {
			X509Certificate x509cert = (X509Certificate)userCert[index_now];
      		// Get version
	      	int version = x509cert.getVersion();
	      	certInfo.put("version", Integer.toString(version));

	      	// Get serial
	      	String serial = x509cert.getSerialNumber().toString();
	      	certInfo.put("serial", serial);

	      	// Get subject
	      	Principal principal = x509cert.getSubjectDN();
	     	String subjectDn = principal.getName();
	     	certInfo.put("subjectDn", subjectDn);

	      	// Get issuer
	      	principal = x509cert.getIssuerDN();
	      	String issuerDn = principal.getName();
	      	certInfo.put("issuerDn", issuerDn);

	      	// Get valid from
	      	String valid_from = x509cert.getNotBefore().toString();
	      	certInfo.put("valid_from", valid_from);

	      	// Get valid until
	      	String valid_until = x509cert.getNotAfter().toString();
	      	certInfo.put("valid_until", valid_until);

	      	// Get certificate signature algorithm
	      	String algorithm = x509cert.getSigAlgName();
	      	certInfo.put("algorithm", algorithm);

	      	Key publickey = x509cert.getPublicKey();
	      	String key_algorithm = publickey.getAlgorithm();
	      	certInfo.put("key_algorithm", key_algorithm);

	      	// Get fingerprint 
	      	try {
		  		byte[] x509data = x509cert.getEncoded();
		  		String md5_fingerprint = StringUtil.getFingerprint(x509data, "MD5");
		  		certInfo.put("md5_fingerprint", md5_fingerprint);
		  		String sha_fingerprint = StringUtil.getFingerprint(x509data, "SHA");
		  		certInfo.put("sha_fingerprint", sha_fingerprint);

	      	} catch (Exception er) {
		  		er.printStackTrace();
	      	}
		}		
	}

	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		sb.append("version:" + get("version"));
		sb.append("\n");
		sb.append("serial:" + get("serial"));
		sb.append("\n");
		sb.append("subjectDn:" + get("subjectDn"));
		sb.append("\n");
		sb.append("issuerDn:" + get("issuerDn"));
		sb.append("\n");
		sb.append("valid_from:" + get("valid_from"));
		sb.append("\n");
		sb.append("valid_until:" + get("valid_until"));
		sb.append("\n");
		sb.append("algorithm:" + get("algorithm"));
		sb.append("\n");
		sb.append("key_algorithm:" + get("key_algorithm"));
		sb.append("\n");
		sb.append("md5_fingerprint:" + get("md5_fingerprint"));
		sb.append("\n");
		sb.append("sha_fingerprint:" + get("sha_fingerprint"));
		return sb.toString();
	}
	
	public String get(String id)
	{
		// FIXME: Consider a case where <id> does not exist
		return (String)certInfo.get(id);
		


	}
}