/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: XML.java,v 1.5 2007/04/25 01:03:33 mountie Exp $ */

package net.kldp.OpenSigner.utils;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.apache.xml.security.utils.IgnoreAllErrorHandler;
import org.apache.xml.security.utils.XMLUtils;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.signature.XMLSignatureException;
import org.apache.xml.security.exceptions.XMLSecurityException;
import org.apache.xpath.XPathAPI;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.cert.X509Certificate;
import java.util.Hashtable;

/**
 * This class implements a simple and naive verification of an OpenSign generated xmldsig document.
 * It is used in the demo setup on www.openoces.org.
 *
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

public class XML {



    public static Document getDocument(String s) throws ParserConfigurationException, IOException, SAXException {
        return getDocument(s.getBytes());
    }

    public static Document getDocument(byte[] bs) throws ParserConfigurationException, IOException, SAXException {
        org.apache.xml.security.Init.init();
        
        javax.xml.parsers.DocumentBuilderFactory dbf;
        dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        dbf.setAttribute("http://xml.org/sax/features/namespaces", Boolean.TRUE);
        Document doc;

        javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
        db.setErrorHandler(new IgnoreAllErrorHandler());
        doc = db.parse(new ByteArrayInputStream(bs));
        return doc;
    }


    public static Hashtable[] getSignatureProperties(Document doc, XMLSignature signature) throws ParserConfigurationException, IOException, SAXException, XMLSignatureException, TransformerException {
        Hashtable[] hts = new Hashtable[3];
        hts[0] = new Hashtable();
        hts[1] = new Hashtable();
        hts[2] = new Hashtable();

        Hashtable ht = hts[0];
        Element nscontext = XMLUtils.createDSctx(doc, "ds", org.apache.xml.security.utils.Constants.SignatureSpecNS);
        if ("http://www.openoces.org/2006/07/signature#".equals(doc.getFirstChild().getNamespaceURI())) {

            for (int i = 0; i < signature.getSignedInfo().getSignedContentLength(); i++) {
                byte[] bs = signature.getSignedInfo().getSignedContentItem(i);
                Document contents = getDocument(bs);

                if ("ToBeSigned".equals(contents.getFirstChild().getAttributes().getNamedItem("Id").getNodeValue())) {
                    NodeList nl = XPathAPI.selectNodeList(contents, "//ds:SignatureProperty", nscontext);
                    Element nsOpenOCEScontext = XMLUtils.createDSctx(doc, "openoces",
                            "http://www.openoces.org/2006/07/signature#");
                    for (int j = 0; j < nl.getLength(); j++) {
                        Node n = nl.item(j);
                        Node nameNode, valueNode;
                        String name, val;

                        nameNode = XPathAPI.selectSingleNode(n, "openoces:Name", nsOpenOCEScontext);
                        valueNode = XPathAPI.selectSingleNode(n, "openoces:Value", nsOpenOCEScontext);

                        name = nameNode.getFirstChild().getNodeValue();
                        val = new String(Base64.decode(valueNode.getFirstChild().getNodeValue().getBytes("UTF-8")), "UTF-8");
                        String encoding = valueNode.getAttributes().getNamedItem("Encoding").getNodeValue();
                        Boolean visibleToSigner = ("yes".equals(valueNode.getAttributes().getNamedItem("VisibleToSigner").getNodeValue())) ? Boolean.TRUE : Boolean.FALSE;

                        ht.put(name, val);
                        hts[1].put(name,encoding);
                        hts[2].put(name,visibleToSigner);
                    }
                }
            }
        } else {
            // FIXME: This is not a secure way to retrieve signed data

            NodeList nl = XPathAPI.selectNodeList(doc, "//ds:SignatureProperties/ds:SignatureProperty", nscontext);
            for (int i = 0; i < nl.getLength(); i++) {
                Node n = nl.item(i);
                Node nameNode, valueNode;
                String name, value;

                nameNode = XPathAPI.selectSingleNode(n, "Name");
                valueNode = XPathAPI.selectSingleNode(n, "Value");

                name = nameNode.getFirstChild().getNodeValue();
                value = new String(Base64.decode(valueNode.getFirstChild().getNodeValue().getBytes("UTF-8")), "UTF-8");
                Boolean visibleToSigner = ("yes".equals(valueNode.getAttributes().getNamedItem("VisibleToSigner").getNodeValue())) ? Boolean.TRUE : Boolean.FALSE;
                ht.put(name, value);
                hts[1].put(name,"n/a");
                hts[2].put(name,visibleToSigner);

            }
        }
        return hts;
    }


    public static XMLSignature verifySignature(Document doc) throws TransformerException, XMLSecurityException, XMLSignatureException, IOException {
        Element nscontext = XMLUtils.createDSctx(doc, "ds", org.apache.xml.security.utils.Constants.SignatureSpecNS);

        Element sigElement = (Element) XPathAPI.selectSingleNode(doc, "//ds:Signature[1]", nscontext);
        XMLSignature signature = new XMLSignature(sigElement, null);
        X509Certificate cert = signature.getKeyInfo().getX509Certificate();
        //FIXME: Check validity of the certificate

        return (signature.checkSignatureValue(cert) ? signature : null);
    }
}
